#include<stdio.h>
#include<stdlib.h>
#include<stdbool.h>
#include<errno.h>


#define OFFSET(base, member) \
    ((char*)&(base.member) - (char*)&(base))

#define MEMBER(member) OFFSET(conf, member)

static bool get_int(char** s, void* p) {
    int* ip = (int*) p;
    char* endptr;
    errno = 0;
    long int val = strtol(*s, &endptr, 10);
    if (errno || val <= 0) return false;
    *ip = val;
    *s = endptr;
    return true;
}

static bool get_double(char** s, void* p) {
    double* dp = (double*) p;
    char* endptr;
    errno = 0;
    double val = strtod(*s, &endptr);
    if (errno) return false;
    *dp = val;
    *s = endptr;
    return true;
}


static struct config{
    int a;
    float b;
} conf ={
    .a = 1,
    .b = .5,
};


static struct Option {
    char name;
    const char* description;
    bool (*input)(char** s, void* p);
    size_t displacement;
} options[] = {
    {'f', "float", get_double, MEMBER(a)},
    {'i', "integer", get_int, MEMBER(b)},
};


typedef struct Option Option;
const unsigned int nofoptions = sizeof(options)/sizeof(options[0]);

int main(int argc, char* argv[]){ 

    --argc; ++argv;

    while (argc > 0 && argv[0][0] == '-') {

	if (argv[0][1] == '-' && argv[0][2] == 0) {
	    --argc; ++argv; break;
	}

	char option_ch = argv[0][1];

	int i;

	for (i = 0; i < nofoptions; ++i) {
	    if (options[i].name == option_ch) break;
	}

	if (i < nofoptions) {
	    Option* option = &options[i];
	    char* args;
	    if (argv[0][2]) {
		args = argv[0] + 2;
	    } else {
		--argc; ++argv;
		if (argc == 0) {
		    fprintf(stderr,"%s: %s",argv[-1], "argument missing"); return 1;
		}
		args = argv[0];
	    }

	    void* valptr = (void*) ((char*) &conf + option->displacement);
	    if (!option->input(&args, valptr) || *args) {
		fprintf(stderr,"%s: %s",argv[0], "invalid argument"); return 1;
	    }
	} 
	--argc; ++argv;
    }
}

